import 'dart:async';
import 'dart:convert';

import 'package:lori_sdk/data/database/interfaces/authResponseDao.dart';
import 'package:lori_sdk/data/models/authResponse.dart';
import 'package:shared_preferences/shared_preferences.dart';

class WebAuthResponseDao implements AuthResponseDao {
  static const _AUTH_RESPONSES_KEY = 'x_auth_responses';

  SharedPreferences db;

  Future open() async {
    if (db == null) {
      db = await SharedPreferences.getInstance();
    }
  }

  Future<void> insertOrUpdate(AuthResponse response) async {
    var storedResponse = await get(response.responseType);
    if (storedResponse == null) {
      return insert(response);
    } else {
      return update(response);
    }
  }

  Future<void> insert(AuthResponse response) async {
    await open();
    List<AuthResponse> responses = await _all();
    responses.add(response);
    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;
    await db.setString(_AUTH_RESPONSES_KEY,
        jsonEncode(responses.map((item) => item.toJson()).toList()));
    return;
  }

  Future<void> update(AuthResponse response) async {
    await open();
    List<AuthResponse> responses = await _all();
    int sameTypeIndex = -1;
    for (var i = 0; i < responses.length; i++) {
      final auth = responses[i];
      if (auth.responseType == response.responseType) {
        sameTypeIndex = i;
      }
    }
    if (sameTypeIndex > -1) {
      responses[sameTypeIndex] = response;
    }
    await db.setString(_AUTH_RESPONSES_KEY,
        jsonEncode(responses.map((item) => item.toJson()).toList()));
    return;
  }

  Future<AuthResponse> get(int responseType) async {
    await open();
    List<AuthResponse> responses = await _all();
    int sameTypeIndex = -1;
    for (var i = 0; i < responses.length; i++) {
      final auth = responses[i];
      if (auth.responseType == responseType) {
        sameTypeIndex = i;
      }
    }
    return sameTypeIndex > -1 ? responses[sameTypeIndex] : null;
  }

  Future<int> delete(int responseType) async {
    await open();
    List<AuthResponse> responses = await _all();
    int sameTypeIndex = -1;
    for (var i = 0; i < responses.length; i++) {
      final auth = responses[i];
      if (auth.responseType == responseType) {
        sameTypeIndex = i;
      }
    }
    if (sameTypeIndex > -1) {
      responses.removeAt(sameTypeIndex);
    }
    await db.setString(_AUTH_RESPONSES_KEY,
        jsonEncode(responses.map((item) => item.toJson()).toList()));
    return responseType;
  }

  Future<void> deleteAll() async {
    await open();
    await db.setString(_AUTH_RESPONSES_KEY, null);
    return;
  }

  Future close() async {}

  Future<List<AuthResponse>> _all() async {
    await open();
    String authResponsesInJson = db.getString(_AUTH_RESPONSES_KEY);
    List<AuthResponse> responses = [];
    if (authResponsesInJson != null) {
      List<Map> responsesMap = List.from(jsonDecode(authResponsesInJson));
      for (var map in responsesMap) {
        responses.add(AuthResponse.fromJson(map));
      }
    }
    return responses;
  }
}

AuthResponseDao getAuthResponseDao() => WebAuthResponseDao();
